import {
    AfterViewInit,
    Component,
    ElementRef,
    OnInit,
    ViewChild
} from '@angular/core';

import { ActivatedRoute } from '@angular/router';

import {
    MatPaginator,
    MatSort,
    MatTableDataSource
} from '@angular/material';

import {
    debounceTime,
    distinctUntilChanged,
    startWith,
    tap,
    delay
} from 'rxjs/operators';

import { merge } from 'rxjs/observable/merge';
import { fromEvent } from 'rxjs/observable/fromEvent';

import { CommentsService } from '../services/comments.service';
import { VideosDataSource } from '../services/videos.datasource';


@Component({
    selector: 'videos',
    templateUrl: './videos.component.html',
    styleUrls: [ './videos.component.css' ]
})
export class VideosComponent implements OnInit, AfterViewInit {
    dataSource: VideosDataSource;

    displayedColumns = [
        'recorded_at',
        'title',
        'game',
        'length'
    ];

    @ViewChild(MatPaginator) paginator: MatPaginator;
    @ViewChild(MatSort) sort: MatSort;
    @ViewChild('input') input: ElementRef;

    constructor(private route: ActivatedRoute,
                private commentsService: CommentsService) { }

    ngOnInit() {
        this.dataSource = new VideosDataSource(this.commentsService);
        this.dataSource.loadVideos('', 'recorded_at', 'desc', 0, 10);
    }

    ngAfterViewInit() {
        this.sort.sortChange.subscribe(() => this.paginator.pageIndex = 0);

        fromEvent(this.input.nativeElement, 'keyup')
            .pipe(
                debounceTime(150),
                distinctUntilChanged(),
                tap(() => {
                    this.paginator.pageIndex = 0;
                    this.loadVideosPage();
                })
            )
            .subscribe();

        merge(this.sort.sortChange, this.paginator.page)
            .pipe(
                tap(() => this.loadVideosPage())
            )
            .subscribe();
    }

    loadVideosPage() {
        this.dataSource.loadVideos(
            this.input.nativeElement.value,
            this.sort.active,
            this.sort.direction,
            this.paginator.pageIndex,
            this.paginator.pageSize);
    }
}
